DEFAULT_POST_SIZE = 32
DEFAULT_TIMEOUT = 180
DEFAULT_RETRY = 5
MIN_TIMEOUT = 5

PYTHON_MAJOR_VERSION = 3

DEFAULT_SC_TIMEOUT = 600
DEFAULT_NB_THREADS = 5

DEFAULT_URL = 'https://api.osskb.org'  # default free service URL
DEFAULT_URL2 = 'https://api.scanoss.com'  # default premium service URL

DEFAULT_API_TIMEOUT = 600

DEFAULT_HFH_RANK_THRESHOLD = 5